#ifndef CONNECT4_H
#define CONNECT4_H

#include <stdbool.h>
#include <stdint.h>

typedef enum {
    PieceNone,
    PiecePlayer1,
    PiecePlayer2,
} Piece;

typedef enum {
    PlacingStatusPiecePlaced,
    PlacingStatusPlayerWon,
    PlacingStatusColumnFull,
    PlacingStatusGridFull,
    PlacingStatusInvalidColumn,
} PlacingStatus;

typedef struct Connect4 {
    int32_t mode;
    Piece current_player;
    int32_t rows;
    int32_t cols;
    Piece **grid;
} Connect4;

Connect4 *connect4_create(int32_t mode, int32_t rows, int32_t cols);
void connect4_destroy(Connect4 *connect4);
Connect4 *connect4_copy(Connect4 *connect4);
void print_grid_separator(int32_t n, char *front, char *rear, char *separator1, char *separator2);
void print_grid_body(Piece **grid, int32_t rows, int32_t cols, int32_t y);
void print_grid(Piece **grid, int32_t rows, int32_t cols);
void connect4_print(Connect4 *connect4);
bool connect4_is_full(Connect4 *connect4);
int32_t count_contiguous_pieces(Connect4 *connect4, int32_t x, int32_t y, int32_t x_direction, int32_t y_direction, Piece piece);
bool connect4_check_win(Connect4 *connect4, int32_t x, int32_t y, Piece piece);
PlacingStatus connect4_place_piece(Connect4 *connect4, Piece piece, int32_t x);
Piece change_player(Piece current_player);
PlacingStatus connect4_play_random(Connect4 *connect4, void (*on_piece_placed)(Connect4 *));
PlacingStatus connect4_play_smart(Connect4 *connect4, void (*on_piece_placed)(Connect4 *));
PlacingStatus connect4_play(Connect4 *connect4, int32_t x, void (*print)(Connect4 *));

#endif
