#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>

#include "Connect4.h"

char *get_game_result(int32_t mode, Piece current_player, PlacingStatus status) {
    if (status == PlacingStatusGridFull) {
        return "It is a draw.";
    }

    if (current_player == PiecePlayer1) {
        return "Player one won!";
    }

    if (current_player == PiecePlayer2 && mode == 3) {
        return "Player two won!";
    }

    return "Computer won!";
}

void print_game_result(int32_t mode, Piece current_player, PlacingStatus status) {
    printf("%s\n", get_game_result(mode, current_player, status));
}

void on_piece_placed(Connect4 *connect4) {
    connect4_print(connect4);
}

void print_help() {
    printf("Usage: puissance4 <mode> <row> <col>\n");
    printf("    mode  specifies the mode: 1 = single player game (random),\n");
    printf("          2 = single player game (AI), 3 = two players game\n");
    printf("    row   specifies the number of rows (>= 4)\n");
    printf("    col   specifies the number of columns (>= 4)\n");
}

int main(int argc, char *argv[]) {
    srand(0);

    if (argc != 4) {
        print_help();
        return EXIT_FAILURE;
    }

    int32_t mode = atoi(argv[1]);
    int32_t rows = atoi(argv[2]);
    int32_t cols = atoi(argv[3]);

    if (mode < 1 || mode > 3 || rows < 4 || cols < 4) {
        print_help();
        return EXIT_FAILURE;
    }

    printf("Board size is %dx%d (rows x col)\n", rows, cols);
    Connect4 *connect4 = connect4_create(mode, rows, cols);
    connect4_print(connect4);
    PlacingStatus status;

    while (true) {
        do {
            printf("Column number? (starts at 1):");
            int32_t x;
            scanf("%d", &x);
            printf("\n");
            status = connect4_play(connect4, x, on_piece_placed);
        } while (status == PlacingStatusColumnFull || status == PlacingStatusInvalidColumn);

        if (status != PlacingStatusPiecePlaced) {
            break;
        }
    }

    print_game_result(mode, connect4->current_player, status);
    connect4_destroy(connect4);

    return EXIT_SUCCESS;
}
